using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;

namespace Virtual_Paging
{
    public class DataStore
    {
        private int absoluteRowCount;
        private int absoluteColCount;
        private int loadedStartRow;
        private int loadedEndRow;
        private int pageRowCount = 200;

        //just used to show when new table is loaded
        public Color ActiveTableColor;
        Color[] colors = new Color[] { Color.LightBlue, Color.LightGoldenrodYellow, Color.PeachPuff, Color.PaleGoldenrod };
        private int colorCt = 0;

        private GridControlBase grid = null;

        public DataTable ActiveTable = null;

        public DataTable CreateATable()
        {
            DataTable dt = new DataTable("MyTable");

            int nCols = this.AbsColCount;

            dt.Columns.Add(new DataColumn("AbsRow"));
            dt.Columns.Add(new DataColumn("PageRow"));
            for (int i = 2; i < nCols; i++)
                dt.Columns.Add(new DataColumn(string.Format("Col{0}", i)));

            if (this.ActiveTable == null)
            {
                //set up headers the first time
                for (int i = 0; i < nCols; ++i)
                {
                    this.grid.Model[0, i + 1 + this.grid.Model.Cols.HeaderCount].Text = dt.Columns[i].ColumnName;
                }
            }
            this.ActiveTableColor = colors[colorCt];
            colorCt = (++colorCt) % 4;
            return dt;

        }

        public bool EnsureDataAvailable(int rowIndex)
        {
            if (rowIndex >= StartRow && rowIndex <= loadedEndRow)
                return true;

            this.ActiveTable = this.LoadAt(rowIndex - this.grid.Model.Rows.HeaderCount - 1);
            return true;
        }

        public DataStore(int pageRowCount, int absoluteRowCount, int absoluteColCount, GridControlBase grid)
        {
            this.absoluteRowCount = absoluteRowCount;
            this.absoluteColCount = absoluteColCount;
            this.pageRowCount = pageRowCount;
            this.grid = grid;
        }

        protected DataTable LoadAt(int pos)
        {
            int start = pos - RowCount / 2;
            if (start < 0)
                start = 0;

            if (start > AbsRowCount)
                return null;

            int nRows = this.RowCount;
            if (start + nRows > AbsRowCount)
                start = AbsRowCount - nRows + 1;

            DataTable dt = CreateATable();

            for (int i = 0; i < nRows; ++i)
            {
                DataRow dr = dt.NewRow();
                dr[0] = i + start;
                dr[1] = i;
                for (int j = 2; j < dt.Columns.Count; j++)
                    dr[j] = string.Format("row{0} col{1}", i + start, j);
                dt.Rows.Add(dr);
            }
            loadedStartRow = start;
            loadedEndRow = start + nRows - 1;
            Console.WriteLine("Loaded data {0}-{1}", loadedStartRow, loadedEndRow);
            return dt;
        }


        public int AbsRowCount
        {
            get { return absoluteRowCount; }
            set { absoluteRowCount = value; }
        }
        public int AbsColCount
        {
            get { return absoluteColCount; }
            set { absoluteColCount = value; }
        }
        public int StartRow
        {
            get { return loadedStartRow; }
            //set{loadedStartRow = value;}
        }
        public int RowCount
        {
            get { return pageRowCount; }
            //set{pageRowCount = value;}
        }
        public int EndRow
        {
            get { return loadedEndRow; }
            //set{loadedEndRow = value;}
        }
    }
}
